@@ -8,6 +8,7 @@
 #'                     'Fitted': plot the fitted cov surface
 #' @param isInteractive an option for interactive plot:
 #'                      TRUE: interactive plot; FALSE: printable plot
+#' @param colSpectrum character vector to be use as input in the 'colorRampPalette' function defining the colouring scheme (default: c('blue','red'))
 #' @param ... other arguments passed into persp3d, persp3D, plot3d or points3D for plotting options
 #'
 #' @examples
@@ -16,12 +17,12 @@
 #' pts <- seq(0, 1, by=0.05)
 #' sampWiener <- Wiener(n, pts)
 #' sampWiener <- Sparsify(sampWiener, pts, 10)
-#' res <- FPCA(sampWiener$yList, sampWiener$tList, 
+#' res <- FPCA(sampWiener$Ly, sampWiener$Lt, 
 #'             list(dataType='Sparse', error=FALSE, kernel='epan', verbose=TRUE))
 #' CreateCovPlot(res)
 #' @export
 
-CreateCovPlot = function(fpcaObj, covPlotType = 'Fitted', isInteractive = FALSE, ...){
+CreateCovPlot = function(fpcaObj, covPlotType = 'Fitted', isInteractive = FALSE, colSpectrum = NULL, ...){
   
   ## Check if plotting covariance surface for fitted covariance surface is proper
   if(covPlotType == 'Fitted'){
@@ -35,10 +36,18 @@ CreateCovPlot = function(fpcaObj, covPlotType = 'Fitted', isInteractive = FALSE,
     }
   }
   
+  if(is.null(colSpectrum)){
+    colFunc = colorRampPalette( c('blue','red') );
+  } else {
+    colFunc = colorRampPalette(colSpectrum)
+  } 
+  
   ## Check if rgl is installed
+  if (isInteractive == FALSE && (!'plot3D' %in% installed.packages()[, ])) {
+    stop("CreateCovPlot requires package 'plot3D'")
+  }
   if(isInteractive == TRUE && is.element('rgl', installed.packages()[,1]) == FALSE){
-    isInteractive = FALSE
-    warning("Interactive plot requires package 'rgl', isInteractive set to be FALSE!")
+    stop("Interactive plot requires package 'rgl'")
   }
   
   ## Define the variables to plot
@@ -54,7 +63,7 @@ CreateCovPlot = function(fpcaObj, covPlotType = 'Fitted', isInteractive = FALSE,
   }
   
   ## Define the plotting arguments
-  args1 <- list( xlab='s', ylab='t', col =  topo.colors(24), zlab = 'C(s,t)', lighting=TRUE)
+  args1 <- list( xlab='s', ylab='t', col =  colFunc(24), zlab = 'C(s,t)', lighting=FALSE)
   if( covPlotType == 'Fitted' ){
     args1$main = 'Fitted covariance surface';
   } else {
